#include "pacula.h"

Map::Map()
{
}


Map::Map(const char*	filename,
		   int			mapIndex,
		   int			cellWidth,
		   int			wallWidth,
		   Point*		offset) :
cellWidth(cellWidth),
wallWidth(wallWidth),
offset(*offset)
{
	load(filename, mapIndex);
}


Map::Map(FILE*	mapFile,
		   int		mapIndex,
		   int		cellWidth,
		   int		wallWidth,
		   Point*	offset) :
cellWidth(cellWidth),
wallWidth(wallWidth),
offset(*offset)
{
	load(mapFile, mapIndex);
}


Map::Map(FILE*	mapFile,
		   int		cellWidth,
		   int		wallWidth,
		   Point*	offset) :
cellWidth(cellWidth),
wallWidth(wallWidth),
offset(*offset)
{
	load(mapFile);
}


void Map::buildHomingMap()
{
	for (int i=0; i<MAP_WIDTH; ++i)
	{
		for (int j=0; j<MAP_HEIGHT; ++j)
		{
			homingMap[i][j] = 0;
		}
	}
	int homingCounter = 999;
	homingMap[ghostHome.x][ghostHome.y] = homingCounter;
	bool keepChecking = true;
	
	while (keepChecking)
	{
		keepChecking = false;
		
		for (int i=0; i<MAP_WIDTH; ++i)
		{
			for (int j=0; j<MAP_HEIGHT; ++j)
			{
				Cell* cell = &cells[i][j];
				if (homingMap[i][j] == homingCounter)
				{
					if (cell->canMoveUp() && (j > 0) && (homingMap[i][j-1] == 0))
					{
						homingMap[i][j-1] = homingCounter-1;
						keepChecking = true;
					}
					if (cell->canMoveDown() && (j < (MAP_HEIGHT-1)) && (homingMap[i][j+1] == 0))
					{
						homingMap[i][j+1] = homingCounter-1;
						keepChecking = true;
					}
					if (cell->canMoveLeft() && (i > 0) && (homingMap[i-1][j] == 0))
					{
						homingMap[i-1][j] = homingCounter-1;
						keepChecking = true;
					}
					if (cell->canMoveRight() && (i < (MAP_WIDTH-1)) && (homingMap[i+1][j] == 0))
					{
						homingMap[i+1][j] = homingCounter-1;
						keepChecking = true;
					}
				}
			}
		}
		--homingCounter;
	}
	
	for (int i=0; i<MAP_WIDTH; ++i)
	{
		for (int j=0; j<MAP_HEIGHT; ++j)
		{
			putchar('0'+((homingMap[i][j])&15));
		}
		putchar('\n');
	}
}

void Map::load(FILE*	mapFile,
				int		mapIndex)
{
	fseek(mapFile, mapIndex * (MAP_HEIGHT * (MAP_WIDTH + 1) + 1), SEEK_SET);
	load(mapFile);
}


void Map::load(const char*	filename,
				int			mapIndex)
{
	FILE* mapFile = fopen(filename, "r");
	load(mapFile, mapIndex);
	fclose(mapFile);
}


void Map::load(FILE* mapFile)
{
	for (int i = 0; i < MAP_HEIGHT; ++i)
	{
		for (int j = 0; j < MAP_WIDTH; ++j)
		{
			char c = fgetc(mapFile);
			putchar(c);
			if ((c >= 65) && (c <= 80))
			{
				cells[j][i] = Cell(c - 65, true);
			}
			else if ((c >= 97) && (c <= 112))
			{
				cells[j][i] = Cell(c - 97, false);
			}
			else if ((c >= 32) && (c <= 47))
			{
				cells[j][i] = Cell(c - 32, false);
				ghostHome = Point(j,i);
				cells[j][i].eatPellet();
			}
			else if ((c >= 48) && (c <= 63))
			{
				cells[j][i] = Cell(c - 48, false);
				pacStart = Point(j,i);
			}
			else
			{
				cells[j][i] = Cell(0, false);
			}
		}
		
		// Get newline character
		fgetc(mapFile);
		putchar('\n');
	}
	
	// Get newline character
	fgetc(mapFile);
	putchar('\n');
}


void Map::draw(BITMAP*	bitmap,
				int		colour)
{
	for (int i = 0; i < MAP_HEIGHT; ++i)
	{
		for (int j = 0; j < MAP_WIDTH; ++j)
		{
			cells[j][i].draw(bitmap,
								&offset,
								Point(j,i).addr(),
								cellWidth,
								wallWidth,
								colour);
		}
	}
}

void Map::drawPellets(BITMAP* bitmap, BITMAP** pellets)
{
	for (int i=0; i<MAP_WIDTH; ++i)
	{
		for (int j=0; j<MAP_HEIGHT; ++j)
		{
			if (cells[i][j].hasPellet() & !cells[i][j].hasPowerPellet())
			{
				blit(pellets[0], bitmap, 0, 0,			(cellWidth+wallWidth)*i+offset.x+wallWidth,
														(cellWidth+wallWidth)*j+offset.y+wallWidth, cellWidth, cellWidth);
			}
		}
	}
}

void Map::drawPowerPellets(BITMAP* bitmap, BITMAP** pellets, int timer)
{
	int i=0, j=0;
	if (cells[i][j].hasPowerPellet())
	{
		blit(pellets[1+timer], bitmap, 0, 0,	(cellWidth+wallWidth)*i+offset.x+wallWidth,
												(cellWidth+wallWidth)*j+offset.y+wallWidth, cellWidth, cellWidth);
	}
	
	j=MAP_HEIGHT-1;
	if (cells[i][j].hasPowerPellet())
	{
		blit(pellets[1+timer], bitmap, 0, 0,	(cellWidth+wallWidth)*i+offset.x+wallWidth,
												(cellWidth+wallWidth)*j+offset.y+wallWidth, cellWidth, cellWidth);
	}
	
	i=MAP_WIDTH-1;
	if (cells[i][j].hasPowerPellet())
	{
		blit(pellets[1+timer], bitmap, 0, 0,	(cellWidth+wallWidth)*i+offset.x+wallWidth,
												(cellWidth+wallWidth)*j+offset.y+wallWidth, cellWidth, cellWidth);
	}
	
	j=0;
	if (cells[i][j].hasPowerPellet())
	{
		blit(pellets[1+timer], bitmap, 0, 0,	(cellWidth+wallWidth)*i+offset.x+wallWidth,
												(cellWidth+wallWidth)*j+offset.y+wallWidth, cellWidth, cellWidth);
	}
}


Cell* Map::getCell(Point* cell)
{
	return &(cells[cell->x][cell->y]);
}


Point* Map::getPacStart(Point* cell)
{
	*cell = pacStart;
	return cell;
}


Point* Map::getGhostHome(Point* cell)
{
	*cell = ghostHome;
	return cell;
}


int Map::getCellWidth()
{
	return cellWidth;
}


int Map::getWallWidth()
{
	return wallWidth;
}


Point* Map::getOffset(Point* offset)
{
	*offset = this->offset;
	return offset;
}
